%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Monte Carlo simulations for section A.7.1 of "Bootstrap Inference in Partially Identified Models: Coverage of the Identified Set"
%   by Federico A. Bugni
%   Submitted to Econometrica
%
%   Monte Carlo simulations for Design 2 and N = 1000
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
disp('Monte Carlo simulations for Design 2 and N = 1000');

% Parameters for the simulation
N = 1000; % dataset sample size.
MCsize = 10000; total number of simulations.
M = 200; % Number of simulations used to approximate the (bootstrap/subsampling/asymptotic approximation) quantile.
verbose = 0; % verbose = 1 turns on some messages in the simulation.

% Vector of subsample sizes.
SSsize = round([N/2;N/3;N/5]); % this row vector should include all subsampling sizes that will be used.

% Options for optimization.
OPTIONS = optimset('LargeScale','off','MaxFunEvals',10^4,'MaxIter',10^4,'Display','off');

% Thresholding sequences:
taus = [0;log(log(N))/sqrt(N);log(N)/sqrt(N);(N^(1/8))/sqrt(N);(N^(1/4))/sqrt(N)];  % this row vector should contain all the values of the thresholding sequences that will be used.

% Desired coverage levels;
ps = [0.95]; % this row vector should contail all the values for coverage levels we are interested in.

% Initialization of matrices:
seedstates = zeros(MCsize,1); % saves the random number seed to enable future replication.
times = zeros(MCsize,1); % time counter (will count the time of each simulation).
% Note: our output will be one matrix of binary values: one if the identified set is covered by the corresponding coverage set and zero otherwise.
% The number of rows of each of these columns is the number of Monte Carlo simulations: MCsize
% The number of columns depends on the procedure.
%  - Bootstrap and asymptotic approximation: Number of significance levels multiplied by the number of threshold sequences.
%  - Subsampling: Number of significance levels, multiplied by the number of threshold sequences, multiplied by number of subsampling sizes.
for DGP = 1 : 4
    eval(['coverage_SS1_DGP',num2str(DGP),'  = zeros(MCsize,size(ps,1)*size(taus,1)*size(SSsize,1));']); % coverage results for subsampling 1.
    eval(['coverage_SS2_DGP',num2str(DGP),'  = zeros(MCsize,size(ps,1)*size(taus,1)*size(SSsize,1));']); % coverage results for subsampling 2.
    eval(['coverage_MYB_DGP',num2str(DGP),'  = zeros(MCsize,size(ps,1)*size(taus,1));']); % coverage results for our bootstrap.
    eval(['coverage_MYAA_DGP',num2str(DGP),' = zeros(MCsize,size(ps,1)*size(taus,1));']); % coverage results for our asymptotic approximation.
end

% Loop over simulations.
for simulation = 1:MCsize;
    disp(['Running MC ', num2str(simulation),' of ',num2str(MCsize)]);
    
    %%%% STEP 0: INITIALIZE %%%%
    % Initialize matrices corresponding to this simulation.
    tic; %Initialize time counter;
    
    % Initalize the matrices corresponding to the simulation.
    % Note: our output will be a matrix of values of the criterion functions.
    % The number of rows of each of these columns is the number of datasets used to construct the (1-alpha)-quantile, which we denoted by M.
    % The number of columns depends on the procedure.
    %  - Bootstrap and asymptotic approximation: Number of threshold sequences.
    %  - Subsampling: Number of threshold sequences, multiplied by number of subsampling sizes.

    % Nunber of simulations used to approximate the (bootstrap/subsampling/asymptotic approximation) quantile.
    SS1 = zeros(M,size(SSsize,1)*size(taus,1)); % Criterion function values for subsampling 1.
    SS2 = zeros(M,size(SSsize,1)*size(taus,1)); % Criterion function values for subsampling 2.
    MYB = zeros(M,size(taus,1)); % Criterion function values for our bootstrap.
    MYAA = zeros(M,size(taus,1)); % Criterion function values for our asymptotic approximation.
    
    % Fix and store the randomness;
    seedstates(simulation) = sum(100*clock); % Generate the seed randomly using the CPU's clock.
    rand('state',seedstates(simulation)); % Fix the random seed.
        
    %%%% STEP 1: CREATE DATA %%%%
    % Dataset creation according to the design.
    for DGP = 1 : 4;
        if verbose == 1;
            disp(['- DGP = ',num2str(DGP)]);
        end
        if DGP  == 1;
            sigma1 = 1; % Std Dev for X1; 
            sigma2 = 1; % Std Dev for X2;
            E1=0; E2=0; % Means of each distribution;
            % Generate auxiliary RVs;
            X1aux = norminv(rand(N,1),0,1);
            X0aux = norminv(rand(N,1),0,1);
            rho = 0; % Correlation between X1 and X2;
            X2aux = rho*X1aux + (sqrt(1-(rho^2)))*X0aux;
            X1 = X1aux * sigma1 + E1; % Generate X1;
            X2 = X2aux * sigma2 + E2; % Generate X2;
        elseif DGP == 2;
            sigma1 = 1; % Std Dev for X1; 
            sigma2 = 1; % Std Dev for X2;
            E1=0; E2=0; % Means of each distribution;
            % Generate auxiliary RVs;
            X1aux = norminv(rand(N,1),0,1);
            X0aux = norminv(rand(N,1),0,1);
            rho = 0.5; % Correlation between X1 and X2;
            X2aux = rho*X1aux + (sqrt(1-(rho^2)))*X0aux;
            X1 = X1aux * sigma1 + E1; % Generate X1;
            X2 = X2aux * sigma2 + E2; % Generate X2;
        elseif DGP == 3;
            sigma1 = 1; % Std Dev for X1; 
            sigma2 = 1; % Std Dev for X2;
            E1=0; E2=0; % Means of each distribution;
            % Generate auxiliary RVs;
            X1aux = norminv(rand(N,1),0,1);
            X0aux = norminv(rand(N,1),0,1);
            rho = -0.5; % Correlation between X1 and X2;
            X2aux = rho*X1aux + (sqrt(1-(rho^2)))*X0aux;
            X1 = X1aux * sigma1 + E1; % Generate X1;
            X2 = X2aux * sigma2 + E2; % Generate X2;
        elseif DGP == 4;
            X1 = tinv(rand(N,1),3); % Generate X1;
            X2 = tinv(rand(N,1),3); % Generate X2;
        end
        
        % Compute relevant features from the data;
        data = [X1,X2];
        
        %%%% STEP 2: COMPUTE THE TRUE VALUE OF THE CRITERION FUNCTION IN SAMPLE  %%%%
        % NOTE: This step is unfeasible in practice, it is only feasible in simulations.
        [Qn_true] = sqrt(N)*(max(mean(data(:,1))-0,0) + max(mean(data(:,2))-0,0));
        
        for m = 1:M
            if verbose == 1;
                disp(['- Running approximation ', num2str(m),' of ',num2str(M),' in MC ', num2str(simulation),'/',num2str(MCsize)]);
            end
            
            %%%% STEP 3.1: IMPLEMENT SUBSAMPLINGS 1 and 2 %%%%   
            if verbose == 1;
                disp(['-- Subsamplings 1 and 2']);
            end
            random_permutation = randperm(N); % permute the numbers 1,2,...,N.
            
            for b=1:size(SSsize,1);
                % Create Subsampling Sample
                % Note: the frequency of each covariate value in the simulated sample is random (random design).
                SSdraws = random_permutation(1:SSsize(b)); % Take the first bn elements of the permutation.
                SSdata = data(SSdraws,:); % Create a subsample sample.
                
                for tau = 1:size(taus,1)
                    [SS1(m,size(taus,1)*(b-1)+tau),SS2(m,size(taus,1)*(b-1)+tau)] = Qn_function_SS(SSdata,data,taus(tau),OPTIONS); % SS1 & SS2 with tau_n = taus(tau).
                end
            end
            
            %%%% STEP 3.2: IMPLEMENT OUR BOOTSTRAP %%%%
            if verbose == 1;
                disp(['-- Bootstrap']);
            end
            % Note: the frequency of each covariate value in the simulated sample is random (random design).
            bootdraws = unidrnd(N,N,1);  % Obtain a bootstrap sample of numbers 1,2,....,N.
            Bdata = data(bootdraws,:); % Create a bootsrap sample.
            
            for tau = 1:size(taus,1)
                MYB(m,tau) = Qn_function_MYB(Bdata,data,taus(tau),OPTIONS);  % Our bootstrap with tau_n = taus(tau).
            end
            
            %%%% STEP 3.2: IMPLEMENT OUR ASYMPTOTIC APPROXIMATION %%%%
            if verbose == 1;
                disp(['-- Asymptotic approximation']);
            end
            % Estimate the variance covariance matrix
            varcov = cov(data);
            Zdata = norminv(rand(1,2))*(varcov^(1/2));
            
            for tau = 1:size(taus,1);
                MYAA(m,tau) = Qn_function_MYAA(Zdata,data,taus(tau),OPTIONS);
            end
        end
        
        %%%% STEP 4: COMPUTE THE EMPIRICAL COVERAGE %%%%
        % Sort the values of each simulated criterion function (we need this to take quantiles)
        % The sorting is done vertically for each column.
        SS1 = sort(SS1); 
        SS2 = sort(SS2);
        MYB = sort(MYB);
        MYAA = sort(MYAA);
        
        % Compute the empirical coverage for each relevant quantile for SS1
        result_row = []; 
        for b=1:size(SSsize,1);
            for tau=1:size(taus,1);
                for p = 1 : size(ps,1);
                    result_row = [ result_row, [ SS1(round(ps(p)*M),size(taus,1)*(b-1)+tau) >= Qn_true ] ];
                end
            end
        end
        eval(['coverage_SS1_DGP',num2str(DGP),'(simulation,:) = result_row;']);
        
        % Compute the empirical coverage for each relevant quantile for SS2
        result_row = [];
        for b=1:size(SSsize,1);
            for tau=1:size(taus,1);
                for p = 1 : size(ps,1);
                    result_row = [ result_row, [ SS2(round(ps(p)*M),size(taus,1)*(b-1)+tau) >= Qn_true ] ];
                end
            end
        end
        eval(['coverage_SS2_DGP',num2str(DGP),'(simulation,:) = result_row;']);
        
        % Compute the empirical coverage for each relevant quantile for MYB
        result_row = [];
        for tau=1:size(taus,1);
            for p = 1 : size(ps,1);
                result_row = [ result_row, [ MYB(round(ps(p)*M),tau) >= Qn_true ] ];
            end
        end
        eval(['coverage_MYB_DGP',num2str(DGP),'(simulation,:) = result_row;']);
        
        % Compute the empirical coverage for each relevant quantile for MYAA
        result_row = [];
        for tau=1:size(taus,1);
            for p = 1 : size(ps,1);
                result_row = [ result_row, [ MYAA(round(ps(p)*M),tau) >= Qn_true ] ];
            end
        end
        eval(['coverage_MYAA_DGP',num2str(DGP),'(simulation,:) = result_row;']);
        
        %%%% STEP 5: STORE THE RESULTS %%%%
        times(simulation)=toc; % stop the time counter and store the result.
        save mc_results simulation coverage_SS1* coverage_SS2* coverage_MYB* coverage_MYAA* seedstates times ps taus SSsize N M MCsize; % Save results.
    end
end