%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Auxiliary function in Monte Carlo simulations for section A.7.2 of "Bootstrap Inference in Partially Identified Models: Coverage of the Identified Set"
%   by Federico A. Bugni
%   Submitted to Econometrica
%
%   Simulations for Design 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Description: This function computes the value of the subsampling criterion function with recentering (SS1)
function [Qn_value] = Qn_function_SS1(SSdata,data,tn,OPTIONS)

% Compute the boundary of the estimated identified set.
N1 = sum(data(:,7)==1); % frequency of data with covariate X=X1;
N2 = sum(data(:,7)==2); % frequency of data with covariate X=X2;
N3 = sum(data(:,7)==3); % frequency of data with covariate X=X3;
N = N1 + N2 + N3;

E1=[mean(data(:,1))*N/N1,mean(data(:,2))*N/N1];
E2=[mean(data(:,3))*N/N2,mean(data(:,4))*N/N2];
E3=[mean(data(:,5))*N/N3,mean(data(:,6))*N/N3];

% We compute the intersection of all (non-parallel) pairs moment inequalities in the model.
% By the structure the criterion function, these constitute adequate starting points for the simulation.
points = [];
points = [points, [norminv(E1(1)-tn);norminv(E2(1)-tn)]]; % 1L & 2L
points = [points, [norminv(E1(1)-tn);norminv(E2(2)+tn)]]; % 1L & 2H
points = [points, [norminv(E1(1)-tn);norminv(E3(1)-tn)-norminv(E1(1)-tn)]]; % 1L & 3L
points = [points, [norminv(E1(1)-tn);norminv(E3(2)+tn)-norminv(E1(1)-tn)]]; % 1L & 3H
points = [points, [norminv(E1(2)+tn);norminv(E2(1)-tn)]]; % 1H & 2L
points = [points, [norminv(E1(2)+tn);norminv(E2(2)+tn)]]; % 1H & 2H
points = [points, [norminv(E1(2)+tn);norminv(E3(1)-tn)-norminv(E1(2)+tn)]]; % 1H & 3L
points = [points, [norminv(E1(2)+tn);norminv(E3(2)+tn)-norminv(E1(2)+tn)]]; % 1H & 3H
points = [points, [norminv(E3(1)-tn)-norminv(E2(1)-tn);norminv(E2(1)-tn)]]; % 2L & 3L
points = [points, [norminv(E3(2)+tn)-norminv(E2(1)-tn);norminv(E2(1)-tn)]]; % 2L & 3H
points = [points, [norminv(E3(1)-tn)-norminv(E2(2)+tn);norminv(E2(2)+tn)]]; % 2H & 3L
points = [points, [norminv(E3(2)+tn)-norminv(E2(2)+tn);norminv(E2(2)+tn)]]; % 2H & 3H

% The value of the criterion function is the result of a constrained maximization.
% The next step expresses these constrains as follows: A*theta <= B;
A = [[-1,0];[1,0];[0,-1];[0,1];[-1,-1];[1,1]];
B = [-norminv(E1(1)-tn);norminv(E1(2)+ tn);-norminv(E2(1)-tn);norminv(E2(2)+ tn);-norminv(E3(1)-tn);norminv(E3(2)+ tn)];

% The next step discards those points that violate the constraints. They will not be considered adequate starting values.
in=zeros(size(points,2),1);
empty = 1;
for j=1:size(points,2);
    if sum( A*points(:,j) > B + eps^0.5) == 0;
        empty = 0;
        in(j)=1;
    end
end

% The next step performs the maximization.
if empty == 0;  % There are suitable starting values (the constraint set is not empty).
    
    % Initalize values for maximization.
    Qn_value = -10; % Initialize arbitrary low value for maximization.
    Aeq = []; % No equality constrains.
    Beq = []; % No equality constrains.
    LB  = [-2,-2]; % Constrains from the parameter space.
    UB  = [2,2]; % Constrains from the parameter space.
    NONLCON = []; % no non-linear constrains.
    
    for j=1:size(points,2);
        if in(j)==1;
            [optimizer,mQn_value,bandera] = fmincon('Qn_eval_MYB',points(:,j),A,B,Aeq,Beq,LB,UB,NONLCON,OPTIONS,SSdata,data,tn);
            % The objective function for SS1 is the same as when we compute the bootstrap, except that bootstrap sample is replaced by subsample sample
            current_value = -mQn_value;

            if current_value >= Qn_value & sum(A*optimizer > B + eps^0.5)==0 & bandera>0;
                Qn_value = current_value;
            elseif Qn_eval_MYB(points(:,j),SSdata,data,tn)>= Qn_value & sum(A*points(:,j) > B + eps^0.5)==0;
                Qn_value = Qn_eval_MYB(points(:,j),SSdata,data,tn);
            end
        end
    end
else
    Qn_value = 0; %  In this case, empty == 1 and there are no suitable starting values (the constraint set is empty)
    disp(['Empty set at SS1 with tn =',num2str(tn)]);
end