%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Auxiliary function in Monte Carlo simulations for section A.7.2 of "Bootstrap Inference in Partially Identified Models: Coverage of the Identified Set"
%   by Federico A. Bugni
%   Submitted to Econometrica
%
%   Simulations for Design 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Description: This function computes the value of the TRUE criterion function.
function [Qn_value] = Qn_function_true(data,E1,E2,E3,OPTIONS);

% We compute the intersection of all (non-parallel) pairs moment inequalities in the model.
% By the structure the criterion function, these constitute adequate starting points for the simulation.
points = [];
points = [points, [norminv(E1(1));norminv(E2(1))]]; % 1L & 2L
points = [points, [norminv(E1(1));norminv(E2(2))]]; % 1L & 2H
points = [points, [norminv(E1(1));norminv(E3(1))-norminv(E1(1))]]; % 1L & 3L
points = [points, [norminv(E1(1));norminv(E3(2))-norminv(E1(1))]]; % 1L & 3H
points = [points, [norminv(E1(2));norminv(E2(1))]]; % 1H & 2L
points = [points, [norminv(E1(2));norminv(E2(2))]]; % 1H & 2H
points = [points, [norminv(E1(2));norminv(E3(1))-norminv(E1(2))]]; % 1H & 3L
points = [points, [norminv(E1(2));norminv(E3(2))-norminv(E1(2))]]; % 1H & 3H
points = [points, [norminv(E3(1))-norminv(E2(1));norminv(E2(1))]]; % 2L & 3L
points = [points, [norminv(E3(2))-norminv(E2(1));norminv(E2(1))]]; % 2L & 3H
points = [points, [norminv(E3(1))-norminv(E2(2));norminv(E2(2))]]; % 2H & 3L
points = [points, [norminv(E3(2))-norminv(E2(2));norminv(E2(2))]]; % 2H & 3H

% The value of the criterion function is the result of a constrained maximization.
% The next step expresses these constrains as follows: A*theta <= B;
A = [[-1,0];[1,0];[0,-1];[0,1];[-1,-1];[1,1]];
B = [-norminv(E1(1));norminv(E1(2));-norminv(E2(1));norminv(E2(2));-norminv(E3(1));norminv(E3(2))];

% The next step discards those points that violate the constraints. They will not be considered adequate starting values.
in=zeros(size(points,2),1);
empty = 1;
for j=1:size(points,2);
    if sum(A*points(:,j) > B + eps^0.5) == 0;
        empty = 0;
        in(j)=1;
    end
end

% The next step performs the maximization.
if empty == 0; % There are suitable starting values (the constraint set is empty).
    
    % Initalize values for maximization.
    Qn_value = -10; % Initialize arbitrary low value for maximization.
    Aeq = []; % No equality constrains.
    Beq = []; % No equality constrains.
    LB  = [-2,-2]; % Constrains from the parameter space.
    UB  = [2,2]; % Constrains from the parameter space.
    NONLCON = []; % no non-linear constrains.
    
    % Constrained maximization starting at each suitable starting point.
    for j=1:size(points,2);
        if in(j)==1;
            [optimizer,mQn_value,bandera] = fmincon('Qn_eval_true',points(:,j),A,B,Aeq,Beq,LB,UB,NONLCON,OPTIONS,data);
            current_value = -mQn_value;
            
            if current_value >= Qn_value & sum(A*optimizer > B + eps^0.5)==0 & bandera>0;
                Qn_value = current_value;
            elseif Qn_eval_true(points(:,j),data)>= Qn_value & sum(A*points(:,j) > B + eps^0.5)==0;
                Qn_value = Qn_eval_true(points(:,j),data);
            end
        end
    end
else %  In this case, empty == 1 and there are no suitable starting values (the constraint set is empty).
    Qn_value = 0;
    disp('Empty set at true');
end