%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Monte Carlo simulations for section A.7.2 of "Bootstrap Inference in Partially Identified Models: Coverage of the Identified Set"
%   by Federico A. Bugni
%   Submitted to Econometrica
%
%   Simulations for Design 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
disp('Monte Carlo simulations for Design 2');

% Parameters for the simulation
MCsize = 1000; % total number of simulations.
M = 200; % Number of simulations used to approximate the (bootstrap/subsampling/asymptotic approximation) quantile.
verbose = 0; % verbose = 1 turns on some messages in the simulation.

S = 100; % Parameter to fix the data sample size;
N1 = 1*S; % Sample size for the first covariate;
N2 = 2*S; % Sample size for the second covariate;
N3 = 3*S; % Sample size for the third covariate;
N = N1 + N2 + N3; % Total sample size;

% Vector of subsample sizes.
SSsize = round([N/2;N/3]); % this row vector should include all subsampling sizes that will be used.

% Options for optimization.
OPTIONS = optimset('LargeScale','off','MaxFunEvals',10^4,'MaxIter',10^4,'Display','off');

% Thresholding sequences:
taus = [log(log(N))/sqrt(N)]; % this row vector should contain all the values of the thresholding sequences that will be used.

% Desired coverage levels;
ps = [0.75;0.9;0.95;0.99]; % this row vector should contail all the values for coverage levels we are interested in.

% Initialization of matrices:
seedstates = zeros(MCsize,1); % saves the random number seed to enable future replication.
times = zeros(MCsize,1); % time counter (will count the time of each simulation).
% Note: our output will be a matrix of binary values: one if the identified set is covered by the corresponding coverage set and zero otherwise.
% The number of rows of each of these columns is the number of Monte Carlo simulations: MCsize
% The number of columns depends on the procedure.
%  - Bootstrap and asymptotic approximation: Number of significance levels multiplied by the number of threshold sequences.
%  - Subsampling: Number of significance levels, multiplied by the number of threshold sequences, multiplied by number of subsampling sizes.
coverage_SS1  = zeros(MCsize,size(ps,1)*size(taus,1)*size(SSsize,1)); % coverage results for subsampling 1.
coverage_SS2  = zeros(MCsize,size(ps,1)*size(taus,1)*size(SSsize,1)); % coverage results for subsampling 2.
coverage_MYB  = zeros(MCsize,size(ps,1)*size(taus,1)); % coverage results for our bootstrap.
coverage_MYAA = zeros(MCsize,size(ps,1)*size(taus,1)); % coverage results for our asymptotic approximation.

% Save the empty matrices.
simulation = 0; % indicates that no simulations have been completed yet.
save mc_results simulation coverage_SS1 coverage_SS2 coverage_MYB coverage_MYAA seedstates times;

% Loop over simulations.
for simulation = 1:MCsize;
    disp(['Running MC ', num2str(simulation),' of ',num2str(MCsize)]);
    
    %%%% STEP 0: INITIALIZE %%%%
    % Initialize matrices corresponding to this simulation.
    tic; %Initialize time counter;
    
    % Initalize the matrices corresponding to the simulation.
    % Note: our output will be a matrix of values of the criterion functions.
    % The number of rows of each of these columns is the number of datasets used to construct the (1-alpha)-quantile, which we denoted by M.
    % The number of columns depends on the procedure.
    %  - Bootstrap and asymptotic approximation: Number of threshold sequences.
    %  - Subsampling: Number of threshold sequences, multiplied by number of subsampling sizes.

    % Nunber of simulations used to approximate the (bootstrap/subsampling/asymptotic approximation) quantile.
    SS1 = zeros(M,size(SSsize,1)*size(taus,1)); % Criterion function values for subsampling 1.
    SS2 = zeros(M,size(SSsize,1)*size(taus,1)); % Criterion function values for subsampling 21.
    MYB = zeros(M,size(taus,1)); % Criterion function values for our bootstrap.
    MYAA = zeros(M,size(taus,1)); % Criterion function values for our asymptotic approximation.
    
    % Fix and store the randomness;
    seedstates(simulation) = sum(100*clock); % Generate the seed randomly using the CPU's clock.
    rand('state',seedstates(simulation)); % Fix the random seed.
    
    %%%% STEP 1: CREATE DATA %%%%
    % Dataset creation according to the design.
    % Observations with X = X1 = (1,0)
    EYW1 = normcdf(-0.5);
    EW1 = normcdf(-0.5)+1-normcdf(0.5); 
    W1 = [rand(N1,1)<EW1];
    Y1 = [rand(N1,1)<EYW1/EW1];
    data1 = [Y1.*W1 , Y1.*W1 + (1-W1)];
    
    % Observations with X = X2 = (0,1)    
    EYW2 = normcdf(-0.5);
    EW2 = normcdf(-0.5)+1-normcdf(0.5);  
    W2 = [rand(N2,1)<EW2];
    Y2 = [rand(N2,1)<EYW2/EW2];
    data2 = [Y2.*W2 , Y2.*W2 + (1-W2)];
    
    % Observations with X = X3 = (1,1)
    EYW3 = normcdf(-1);
    EW3 = normcdf(-1)+1-normcdf(0.5);
    W3 = [rand(N3,1)<EW3];
    Y3 = [rand(N3,1)<EYW3/EW3];
    data3 = [Y3.*W3 , Y3.*W3 + (1-W3)];
      
    % Combine the data from the three covariate values.
    % The data is a matrix with: N rows and 7 columns: 
    % Columns are: [ Y*W*1[X=X1], (Y*W+(1-W))*1[X=X1], Y*W*1[X=X2], (Y*W+(1-W))*1[X=X2] , Y*W*1[X=X3], (Y*W+(1-W))*1[X=X3] , 1[X=X1] + 2[X=X2] + 3[X=X3]];    
    data = [[data1,zeros(size(data1,1),4),ones(size(data1,1),1)];...
            [zeros(size(data2,1),2),data2,zeros(size(data2,1),2),2*ones(size(data2,1),1)];...
            [zeros(size(data3,1),4),data3,3*ones(size(data3,1),1)]];
    
    %%%% STEP 2: COMPUTE THE TRUE VALUE OF THE CRITERION FUNCTION IN SAMPLE  %%%%
    % NOTE: This step is unfeasible in practice, it is only feasible in simulations.
    
    % Compute parameters used in the calculation.
    E1 = [EYW1,EYW1+(1-EW1)];
    E2 = [EYW2,EYW2+(1-EW2)];
    E3 = [EYW3,EYW3+(1-EW3)];
    
    % Compute the unobserved true 
    [Qn_true] = Qn_function_true(data,E1,E2,E3,OPTIONS);
    
    %%%% STEP 3: IMPLEMENT EACH APPROXIMATION METHOD %%%%
    for m = 1:M
        if verbose == 1;
            disp(['- Running approximation ', num2str(m),' of ',num2str(M),' in MC ', num2str(simulation),'/',num2str(MCsize)]);
        end
        
        %%%% STEP 3.1: IMPLEMENT SUBSAMPLINGS 1 and 2 %%%%   
        if verbose == 1;
            disp(['-- Subsamplings 1 and 2']);
        end
        random_permutation = randperm(N); % permute the numbers 1,2,...,N.
        
        for b=1:size(SSsize,1);
            % Create Subsampling Sample
            % Note: the frequency of each covariate value in the simulated sample is random (random design).
            SSdraws = random_permutation(1:SSsize(b)); % Take the first bn elements of the permutation.
            SSdata = data(SSdraws,:); % Create a subsample sample.
            
            % Compute parameters for the subsample computation,
            E1hat = mean(data1);
            E2hat = mean(data2);
            E3hat = mean(data3);
            
            for tau = 1:size(taus,1)
                SS1(m,size(taus,1)*(b-1)+tau) = Qn_function_SS1(SSdata,data,taus(tau),OPTIONS); % SS1 with tau_n = taus(tau).
                SS2(m,size(taus,1)*(b-1)+tau) = Qn_function_SS2(SSdata,data,taus(tau),OPTIONS); % SS2 with tau_n = taus(tau).
            end
        end
        
        %%%% STEP 3.2: IMPLEMENT OUR BOOTSTRAP %%%%
        if verbose == 1;
            disp(['-- Bootstrap']);
        end
        % Note: the frequency of each covariate value in the simulated sample is random (random design).
        bootdraws = unidrnd(N,N,1);  % Obtain a bootstrap sample of numbers 1,2,....,N.
        Bdata = data(bootdraws,:); % Create a bootsrap sample.
        
        for tau = 1:size(taus,1)
            MYB(m,tau) = Qn_function_MYB(Bdata,data,taus(tau),OPTIONS);  % Our bootstrap with tau_n = taus(tau).
        end
        
        %%%% STEP 3.2: IMPLEMENT OUR ASYMPTOTIC APPROXIMATION %%%%
        if verbose == 1;
            disp(['-- Asymptotic approximation']);
        end
        % Estimate the variance covariance matrix
        % Note: by definition, there will be no cross correlation for different covariance values.
        sigma1 = cov(data1)^(1/2);
        sigma2 = cov(data2)^(1/2);
        sigma3 = cov(data3)^(1/2);
        
        % Create data from normal draws.
        % Note: the frequency of each covariate value in the simulated sample is random (random design).
        % for simplicity the covariate frequency is taken from the bootstrap frequency draws.        
        Zdata1 = norminv(rand(sum(Bdata(:,7)==1),2))*sigma1;
        Zdata2 = norminv(rand(sum(Bdata(:,7)==2),2))*sigma2;
        Zdata3 = norminv(rand(sum(Bdata(:,7)==3),2))*sigma3;
        
        % Combine the simulated data from the three covariate values.
        Zdata = [[Zdata1,zeros(size(Zdata1,1),4),ones(size(Zdata1,1),1)];...
                [zeros(size(Zdata2,1),2),Zdata2,zeros(size(Zdata2,1),2),2*ones(size(Zdata2,1),1)];...
                [zeros(size(Zdata3,1),4),Zdata3,3*ones(size(Zdata3,1),1)]];
   
        for tau = 1:size(taus,1)
            MYAA(m,tau) = Qn_function_MYAA(Zdata,data,taus(tau),OPTIONS);  % Our asymptotic approximation with tau_n = taus(tau).
        end        
    end
    
    %%%% STEP 4: COMPUTE THE EMPIRICAL COVERAGE %%%%
    % Sort the values of each simulated criterion function (we need this to take quantiles)
    % The sorting is done vertically for each column.
    SS1 = sort(SS1); 
    SS2 = sort(SS2);
    MYB = sort(MYB);
    MYAA = sort(MYAA);
    
    % Compute the empirical coverage for each relevant quantile for SS1
    result_row = [];    
    for j = 1 : size(SS1,2);
        for p = 1 : size(ps,1);
            result_row = [ result_row, [ SS1(round(ps(p)*M),j) >= Qn_true ] ];
        end
    end
    coverage_SS1(simulation,:) = result_row;
    
    % Compute the empirical coverage for each relevant quantile for SS2
    result_row = [];
    for j = 1 : size(SS2,2);
        for p = 1 : size(ps,1)
            result_row = [ result_row , [ SS2(round(ps(p)*M),j) >= Qn_true ] ];
        end
    end
    coverage_SS2(simulation,:) = result_row;
    
    % Compute the empirical coverage for each relevant quantile for MYB
    result_row = [];
    for j = 1 : size(MYB,2);
        for p = 1 : size(ps,1)
            result_row = [ result_row , [ MYB(round(ps(p)*M),j) >= Qn_true ] ];
        end
    end
    coverage_MYB(simulation,:) = result_row;
    
    % Compute the empirical coverage for each relevant quantile for MYAA
    result_row = [];
    for j = 1 : size(MYAA,2);
        for p = 1 : size(ps,1)
            result_row = [ result_row , [ MYAA(round(ps(p)*M),j) >= Qn_true ] ];
        end
    end
    coverage_MYAA(simulation,:) = result_row;
    
    %%%% STEP 5: STORE THE RESULTS %%%%
    times(simulation)=toc; % stop the time counter and store the result.
    save mc_results simulation coverage_SS1 coverage_SS2 coverage_MYB coverage_MYAA seedstates times -append; % Save intermediate results.
end